
xVersion" 2012.10.10.1"

module spi1

  #document spi1
    //
    // SPI1 interface
    //
  #endDoc

private

  uses
    hw_bit_defs
    hw_interface
    hw_config
    queues
    utils
  endUses


  // //////////////////////////////////////////////////////////////////////////

  queue         spi1.rx 32
  variable      spi1.rx.expectCount
  variable      spi1.rx.Receiving

  queue         spi1.tx 32
  variable      spi1.tx.expectCount
  variable      spi1.tx.done

  varInterface  spi1.rx.expectCount
  varInterface  spi1.rx.Receiving
  varInterface  spi1.tx.expectCount
  varInterface  spi1.tx.done

  // //////////////////////////////////////////////////////////////////////////

  : spi1.setup ( -- )

    bitClear IFS0 SPI1IF                    // Clear IF
    // bitClear IFS0 SPI1EIF                   // Clear error IF
    bitClear IEC0 SPI1IE                    // disable interrupts
    // bitClear IEC0 SPI1EIE                   // disable interrupts
    [
      PPRE_4
      SPRE_7      or
      MSTEN       or
      CKE         or
      SMP         or
      MODE16  not or
      SPIEN       or
      literal
    ] reg! SPI1CON1                         // Configure device
    spi1.rx.clear
    spi1.tx.clear
    bitSet IEC0 SPI1IE                      // enable interrupts
    // bitSet IEC0 SPI1EIE                     // enable interrupts
  ;


  // //////////////////////////////////////////////////////////////////////////

  : spi1.startLow ( n -- )

    spi1.tx.done.clear
    Reg! SPI1BUF
  ;


  // //////////////////////////////////////////////////////////////////////////

  : spi1.start ( -- )

    spi1.txFillCount @                      // See if something in tx buffer
    0>
    if                                      // Yes, borrow a byte from there
      spi1.tx.read >r
      spi1.tx.expectCount 1-!               // One less to expect
    else                                    // No, use a dummy byte
      $5555 >r
    endIf
    // Flash1CsOn                           // make sure CS is active
    r> spi1.startLow
  ;

  // //////////////////////////////////////////////////////////////////////////

  : spi1.transmit ( n -- )

    spi1.rx.hasRoom 0 >
    if
      spi1.tx.write
      spi1.start
    endif
  ;


  // //////////////////////////////////////////////////////////////////////////

  : _SPI1Interrupt ( -- )

    interruptProlog                         // Setup for interrupt processing
    try
      bitClear IFS0 SPI1IF                  // Clear IF
      // Check for RX work to do

      reg@ SPI1BUF                          // Read data, must be done always to avoid overflows
      spi1.rx.expectCount@ 0>               // Do we expect data?
      if
        spi1.rx.expectCount 1-!             // One less to expect
        spi1.rx.write                       // Yes, write it
      else
        drop                                // No, discard it
      endif

      // Check for TX work to do
      spi1.tx.hasData 0>                    // Do we have data to transmit?
      if
        spi1.tx.read                        // Read data from queue
        Reg! SPI1BUF                        // Send it
        spi1.tx.expectCount 1-!             // One less to expect
      else
        spi1.tx.expectCount@ 0=
        if
          spi1.tx.done.set                  // Flag transmissions to be done
        endif
        spi1.rx.expectCount@ 0>             // Check if we must do a dummy xmiy
        if
          $5555                             // Dummy tx data
          Reg! SPI1BUF                      // Send it
        endif
      endif

    finally
      interruptEpilog                       // Restore foreground environment
    endTry

  interrupt; used global                    // Interrupt handler must end in interupt;
                                            // so that retfie will be compiled for ret,
                                            // it must be set used to not be optimized
                                            // out and global for the linker to be able
                                            // to find it.


  // //////////////////////////////////////////////////////////////////////////

  : _SPI1ErrInterrupt ( -- )

    interruptProlog                         // Setup for interrupt processing
    try
      spi1.setup                            // Reset the hardware
    finally
      interruptEpilog                       // Restore foregrounf environment
    endTry

  interrupt; used global                    // Interrupt handler must end in interupt;
                                            // so that retfie will be compiled for ret,
                                            // it must be set used to not be optimized
                                            // out and global for the linker to be able
                                            // to find it.


  // //////////////////////////////////////////////////////////////////////////

public

  // //////////////////////////////////////////////////////////////////////////

  export  spi1.setup              (                 --        ) // SPI 1, Module initialization

  export  spi1.rx.clear           (                 --        ) // SPI 1, Flush receive buffer
  export  spi1.rx.hasData         (                 -- b      ) // SPI 1, Get amount of RX bytes available
  export  spi1.rx.Read            (                 -- b      ) // SPI 1, Read a received byte
  export  spi1.tx.hasRoom         (                 -- b      ) // SPI 1, Query room in TX buffer
  export  spi1.tx.write           ( b               --        ) // SPI 1, Write a byte to be transmitted
  export  spi1.tx.clear           (                 --        ) // SPI 1. Flush transmit buffer
  export  spi1.rx.expectCount!    ( b               --        ) // SPI 1, Set the number of RX bytes to expect
  export  spi1.rx.expectCount@    (                 -- b      ) // SPI 1, Get the number of RX bytes to expect
  export  spi1.tx.expectCount!    ( b               --        ) // SPI 1, Set the number of TX bytes to expect
  export  spi1.tx.expectCount@    (                 -- b      ) // SPI 1, Get the number of TX bytes to expect
  export  spi1.tx.done?           (                 -- f      ) // SPI 1, Check to see if all transmission done
  export  spi1.start              (                 --        ) // SPI 1, Start the interrupt mechanism

  // //////////////////////////////////////////////////////////////////////////

endModule

