// test.c
//
// Some C functions to test C calling from 4th code
//
// A C function callable from 4th should have the following proto type
//
//  int test_func( int ArgCount, ...)
//
// In 4th the following construction needs to be written out
//
//   push_arguments       // Push the ... bit for the C function
//                        // onto the 4th data stack, there should
//                        // be at least <ArgCount> pushes
//   <ArgCount>           // Specify the number of items pushed
//   c_exec test_func     // c_exec will read the following token
//                        // from the input stream and will thunk
//                        // it into a C call by building a stack
//                        // frame and filling that with data from
//                        // the 4th data stack.
//
// c_exec is speciefied in 4th to have thwe stack effect
//
//   ( ? ArgCount -- Result )
//
// Where ? maps onto the elipsis (...) in the C function. ArgCount
// to the ArgCount argument of the C function and Result to the
// value returned by the C function.


  #include <stdarg.h>               // Needed to access the unspecified
                                    // arguments represented by ...
  #include <stdio.h>                // For printf()
  #include <stdlib.h>               // For malloc() stuff
  #include <setjmp.h>               // For longjm() stuff
  #include "test.h"


  // --------------------------------------------------------------------------

  // Try exception stuff, IIRC the outer do .. while( 0) loop was there to
  // allow for the end_try to be followed by a semi-colon. GCC however seems
  // to allow for the semi-colon to be straight after a closing curly brace,
  // and so the loop is not needed here.

  #define c_try         /* do{ */ jmp_buf exc_env; switch( setjmp( exc_env) ){ case 0: while( 1){
  #define c_except( x)  break; case x:
  #define c_finally     break; } default:
  #define c_endtry      } /* } while( 0) */
  #define c_raise( x)   longjmp( exc_env, x)


  // --------------------------------------------------------------------------

  enum {
    eFoo = 1,           // First value should be > 0
    eBar,
    eBaz,
  };


  int ttt = 15;

  // --------------------------------------------------------------------------

  static void test_man( void)
  // A manual implementation of what the exception macros should build
  {
    // do
    // {
      jmp_buf exc_env;

      switch( setjmp( exc_env))
      {
        case 0:
          while( 1)
          {
            printf( "in try\n");
            longjmp( exc_env, 1); // raise 1
            printf( "never here\n");
            break;
        case 1:
          printf( "exception 1\n");
          break;
        case 2:
          printf( "exception 2\n");
          break;
          }
        default:
          printf( "finally\r");
      }
    // }
    // while( 0);
  }


  // --------------------------------------------------------------------------

  static void test_exceptions( void)
  {
    c_try
    {
      printf( "in c_try\n");
      c_raise( eBar);
      printf( "we never get here\n");
    }
    c_except( eFoo)
    {
      printf( "Got Foo exception\n");
    }
    c_except( eBar)
    {
      printf( "Got Bar exception\n");
    }
    c_except( eBaz)
    {
      printf( "Got Baz exception\n");
    }
    c_finally
    {
      printf( "...et in arcadia ego\n");
    }
    c_endtry;
  }


  // --------------------------------------------------------------------------

  extern int CEventCount( void);    // It's a bit naive to expect this to work ...
                                    // Before calling this a forth environment
                                    // must be set up

  int __attribute__((persistent)) Counter;
  jmp_buf env;


  // --------------------------------------------------------------------------

  static int test_call( int aap)    // Just some additional test function
  {
    enum { BUF_SIZE = 128};

    unsigned int * p;
    unsigned int   i;

    p = calloc( BUF_SIZE, sizeof( int));

    for( i = 0; i < BUF_SIZE; i++)
    {
      p[ i] = i;
    }

    free( p);
    p = NULL;

    longjmp( env, 1);         // This will exit from the function

    return aap + 1;           // The return value will not be set
  }


  // --------------------------------------------------------------------------

  int test_func( int ArgCount, ...) // The function to be called from 4th
  {
    va_list ap;
    int i, sum;
    // int cnt;

    unsigned long int a = 0x01234567;
    unsigned long int b = 0x89abcdef;
    unsigned long int c;

    float aap  = 1e4;
    float noot = 3.14789;
    float mies;

    c = b / a;

    mies = aap / noot;

    if( setjmp( env) == 0)
    {

      printf( "test_func( %d), and ttt = %d, and mies=%f\n", Counter++, ttt, mies);

      va_start( ap, ArgCount);

      sum = 0;
      for( i = 0; i < ArgCount; i++)
      {
        sum += va_arg( ap, int);
      }

      va_end( ap);

      sum = test_call( sum);  // The assignment will not be executed
    }


    // This wont fork FTTB
    //
    // cnt = CEventCount(); // Calls into some 4th stuff
    // printf( "  ... BTW ... 4th has %d events\n", cnt);

    test_man();
    test_exceptions();

    return sum;
  }


  // --------------------------------------------------------------------------


  // int main( void)
  // {
  //   for(;;)
  //   {
  //     test_func( 3, 6, 7);
  //   }
  //   return 0;
  // }

  // --------------------------------------------------------------------------


