
xVersion" 2012.10.10.1"

module uart1

private

  uses
    hw_bit_defs
    hw_config
    utils
    queues
  endUses


  queue uart1.rx 256
  queue uart1.tx 256


  // //////////////////////////////////////////////////////////////////////////

  : _U1RXInterrupt ( -- )           #// uart 1 RX interrupt

    // breakPoint U1RxInterrupt        // Halt debugger

    interruptProlog                 // Setup for interrupt processing
    try
      begin                         // Read all available RX data
        reg@ U1STA
        [ 1 URXDA << literal ]
        maskCheck
      while
        reg@ U1RXREG                // Read recieve register
        uart1.rx.write              // and write data into queue
      repeat
      bitClear IFS0 U1RXIF          // Clear interrupt cause
    finally
      interruptEpilog               // Restore foreground environment
    endTry

  interrupt; used global            // Interrupt handler must end in interupt;
                                    // so that retfie will be compiled for ret,
                                    // it must be set used to not be optimized
                                    // out and global for the linker to be able
                                    // to find it.

  // //////////////////////////////////////////////////////////////////////////

  : _U1TXInterrupt ( -- )           #// uart 1 TX interrupt

    // breakPoint U1TxInterrupt        // Halt debugger

    interruptProlog                 // Setup for interrupt processing
    try
   // bitClear IFS0 U1TXIF          // Clear interrupt cause
      uart1.tx.hasData              // Check if TX data available
      if
        uart1.tx.read               // Yes, read it
        reg! U1TXREG                // and send it
      else
        bitClear IEC0 U1TXIE        // No, disable TX interrupts
      endif
    finally
      interruptEpilog               // Restore foreground environment
    endTry

  interrupt; used global


  // //////////////////////////////////////////////////////////////////////////

  defer uart1.cleanup

  : _U1ErrInterrupt ( -- )          #// uart 1 error interrupt

    breakPoint U1ErrInterrupt       // Halt debugger

    interruptProlog                 // Setup for interrupt processing
    try
      uart1.cleanup                 // Clear interrupt cause
    finally
      interruptEpilog               // Restore foreground environment
    endTry

  interrupt; used global


  // //////////////////////////////////////////////////////////////////////////

  : uart1.setup ( -- )

    bitClear  IEC0    U1RXIE                // disable RX    interrupts
    bitClear  IEC0    U1TXIE                // disable TX    interrupts
    bitClear  IEC4    U1ERIE                // disable error interrupts

    uart1.rx.clear                          // Clear queues
    uart1.tx.clear

    begin                                   // Read all available RX data
      reg@ U1STA
      [ 1 URXDA << literal ]
      maskCheck
    while
      reg@drop U1RXREG                      // and discard it
    repeat

    bitclear  U1STA   OERR                  // Clear overrun error
    bitclear  U1STA   FERR                  // Clear framing error
    bitclear  U1STA   PERR                  // Clear parity  error

    bitClear  IFS0    U1RXIF                // Clear RXIF flag
    bitClear  IFS0    U1TXIF                // Clear TXIF flag
    bitClear  IFS4    U1ERIF                // Clear ERIF flag

    bitset    U1MODE  UARTEN                // Enable UART
    bitSet    U1STA   UTXEN                 // Emable transmitter

    bitSet    IEC4    U1ERIE                // enable error interrupts
    bitSet    IEC0    U1RXIE                // enable RX    interrupts
    bitSet    IEC0    U1TXIE                // enable TX    interrupts
  ;
    implements uart1.cleanup


  // //////////////////////////////////////////////////////////////////////////

  : uart1.write ( c -- )

    uart1.tx.write
    bitSet IEC0 U1TXIE
  ;

  alias uart1.tx.hasRoom  uart1.hasRoom
  alias uart1.rx.read     uart1.read
  alias uart1.rx.hasData  uart1.hasData


  // //////////////////////////////////////////////////////////////////////////

  : uart1.write$ram ( $ram -- )

    ?do
      adup c@ uart1.write
      1a+
    loop
    aDrop
  ;

  // //////////////////////////////////////////////////////////////////////////

  : uart1.writeRom ( prom  -- )

    #// Accept data from a ROM location

    RomPtr!
    Romc@++
    0 ?do
      Romc@++
      uart1.write
    loop
  ;

  // //////////////////////////////////////////////////////////////////////////

public

  export  uart1.setup               (       --    )

  export  uart1.hasData             (       -- w  )
  export  uart1.hasRoom             (       -- w  )
  export  uart1.read                (       -- b  )
  export  uart1.write               ( b     --    )

  export  uart1.rx.overflows        (       -- w  )
  export  uart1.rx.underflows       (       -- w  )
  export  uart1.tx.overflows        (       -- w  )
  export  uart1.tx.underflows       (       -- w  )

  export  uart1.write$ram           ( $ram  --    )
  export  uart1.writeRom            ( prom  --    )

endModule

